"""
Example from Week 4 Review Session covering practice with file io and
dictionaries.

This program ...
"""

def get_substitution_dict():
    """
    Factors out a search dictionary for quick access without adding
    a global variable for good program design (within the scope of material we
    know so far).

    Returns:
        (dict) - str -> str dictionary mapping search key strings to 
                        replacement strings for substitutions in text.
    """
    SEARCH_DICT = {
       'car': 'cat',
       'self-driving': 'uncontrollably-swerving',
       'features': 'features (definitely not bugs)',
       'autonomous': 'eerily self-aware',
       'poll': 'psychic reading',
       'successfully': 'suddenly',
       'certain': '(that one)',
       'benefits': 'coffees',
       'costs': 'costs (in DogeCoin)',
       'capabilities': 'skill points',
       'explains': 'Tries to explain',
       'Explains': 'Tries to explain',
       'regulations': 'laws that don\'t make sense',
       'vows to': 'probably won\'t',
       'promises': 'overcommits',
       'an unknown number': 'probably hundreds',
       'phone': 'Pokedex',
       'map': 'Pokemon Go',
       'leader': 'elf lord',
       'president': 'elf lord',
       'ancient': 'haunted',
       'surprising': 'surprising (but not to me)',
       'email': 'poem',
       'road': 'Pokemon route',
       'tweet': 'poem',
       'market': 'Etsy market',
       'years': 'minutes',
       'money': 'bitcoins',
       'futuristic': 'fairytale',
       'in-depth': 'automatically-generated',
       'report': 'diary entry',
       'analysis': 'results of a heated debate',
       'latest': 'final',
       'photo': 'meme',
       'post': 'meme',
       'article': 'meme'
    }
    return SEARCH_DICT


def replace_words(filename, search, replacement):
    """
    Replaces all instances of a `search` keyword (str) with `replacement` (str)
    in the file corresponding to the given `filename` (str).

    Assumes that `filename` exists.

    Write the results to a new file called replaced_<filename>, leaves
    the original file unchanged.

    Returns:
        (int) - number of words that were replaced.
    """
    # Similar to behavior in MP3 tabs_to_spaces, where file is created
    # in form 'spaced_<filename>
    new_filename = ''
    replacements = 0
    # 1. Open up the file corresponding to the given `filename` str.
    # 2. ...
    return replacements


def replace_all_words(filename, substitutions):
    """
    Replaces all instances of each search key in `substitutions`
    with the corresponding replacement value
    in the file corresponding to the given `filename` (str).

    Assumes that `filename` exists.

    Write the results to a new file called replaced_<filename>, leaves
    the original file unchanged.

    Returns:
        (int) - number of words that were replaced.
    """
    # Similar to behavior in MP3 tabs_to_spaces, where file is created
    # in form 'spaced_<filename>'
    new_filename = 'TODO'
    replacements = 0
    # 1. Open up the file corresponding to the given `filename` str.
    # search value for a key can be found with dict[key] syntax
    with open(filename, 'r') as f:
        # 2. Process the file...
        # lines = f.read() # or lines = f.readlines()
        newf = open(new_filename, 'w')
        for line in f:
            # line: 'Self-driving cars are no longer a futuristic idea...\n'
            # Loop over the substitutions dictionary:
            #       { 'car': 'cat', ..., 'futuristic': 'fairytale', ...}
            # for <key> in <dict>: (implicitly loops through all _keys_ in the dictionary)
            for search_key in substitutions:
                # search == 'car'
                replacement = substitutions[search_key]  # e.g. 'cat'
                # 3. Count the number of occurrences of `search` in `line`
                #    and update replacements accordingly
                occurrences = line.count(search_key)
                if occurrences > 0:
                    replacements += occurrences
                    # 4. Replace any search with replacement
                    line = line.replace(search_key, replacement)
                    # e.g. 'Self-driving cars are no longer...'.replace('car', 'cat')
                    #      'Self-driving cats are no longer...'.replace('car', 'cat')
                    # Note that replace will replace _all_ instances, so not additional loop needed

            # 5. Write the new line to the new file
            # first line: 'Self-driving cats are no longer a fairytale idea...\n'
            newf.write(line)

        # close the file that was opened since we didn't use
        # with to open it
        newf.close()
        return replacements


if __name__ == '__main__':
    # Commented out the first version we did with a single search/replace
    # replacements = replace_words('self_driving_cars.txt', 'car', 'cat')
    # Second (better) version that generalizes a collection of substitutions
    # inspired by the XKCD substitution comics.
    substitutions = get_substitution_dict()
    filename = 'self_driving_cars.txt'
    replacements = replace_all_words(filename, substitutions)
    print(f'Successfully substituted {replacements} words in {filename},')
    print(f'saving the results to replaced_{filename}!')
